@@ -10,11 +10,12 @@
 #'                    used for fitting the GLM.
 #' @param obs_data    Data on which the model is fit.
 #' @param j           Integer specifying the index of the covariate.
+#' @param model_fits  Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #'
 #' @return            Fitted model for the covariate at index \eqn{j}.
 #' @keywords internal
 
-fit_glm <- function(covparams, covlink = NA, covfam, obs_data, j){
+fit_glm <- function(covparams, covlink = NA, covfam, obs_data, j, model_fits){
   # Get model parameters
   covmodels <- covparams$covmodels
   if (!is.null(covparams$covlink)){
@@ -28,11 +29,20 @@ fit_glm <- function(covparams, covlink = NA, covfam, obs_data, j){
   }
 
   # Fit GLM for covariate using user-specified formula
-  fit <- stats::glm(stats::as.formula(paste(covmodels[j])), family = eval(parse(text = famtext)),
-             data = obs_data, y = TRUE)
+  if (!is.null(covparams$control) && !is.na(covparams$control[j])){
+    fit <- stats::glm(stats::as.formula(paste(covmodels[j])), family = eval(parse(text = famtext)),
+                      data = obs_data, y = TRUE, control = covparams$control[j])
+  } else {
+    fit <- stats::glm(stats::as.formula(paste(covmodels[j])), family = eval(parse(text = famtext)),
+                      data = obs_data, y = TRUE)
+  }
+
   fit$rmse <- add_rmse(fit)
   fit$stderrs <- add_stderr(fit)
-  fit <- trim_glm(fit)
+  fit$vcov <- add_vcov(fit)
+  if (!model_fits){
+    fit <- trim_glm(fit)
+  }
   return (fit)
 }
 
@@ -45,15 +55,27 @@ fit_glm <- function(covparams, covlink = NA, covfam, obs_data, j){
 #'                    model statement, family, link function, etc.).
 #' @param obs_data    Data on which the model is fit.
 #' @param j           Integer specifying the index of the covariate.
+#' @param model_fits  Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #'
 #' @return            Fitted model for the covariate at index \eqn{j}.
 #' @keywords internal
 
-fit_multinomial <- function(covparams, obs_data, j){
+fit_multinomial <- function(covparams, obs_data, j, model_fits){
   covmodels <- covparams$covmodels
-  fit <- nnet::multinom(stats::as.formula(paste(covmodels[j])), data = obs_data)
+  if (!is.null(covparams$control) && !is.na(covparams$control[j])){
+    args <- c(list(formula = stats::as.formula(paste(covmodels[j])),
+                   data = obs_data), trace = FALSE, covparams$control[j])
+    fit <- do.call(nnet::multinom, args = args)
+  } else {
+    fit <- nnet::multinom(stats::as.formula(paste(covmodels[j])),
+                          data = obs_data, trace = FALSE)
+  }
+
   fit$stderr <- add_stderr(fit)
-  fit <- trim_multinom(fit)
+  fit$vcov <- add_vcov(fit)
+  if (!model_fits){
+    fit <- trim_multinom(fit)
+  }
   return (fit)
 }
 
@@ -70,11 +92,13 @@ fit_multinomial <- function(covparams, obs_data, j){
 #' @param covname     Name of the covariate at index \eqn{j}.
 #' @param obs_data    Data on which the model is fit.
 #' @param j           Integer specifying the index of the covariate.
+#' @param model_fits  Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #' @return            Fitted model for the covariate at index \eqn{j}.
 #' @keywords internal
 #' @import data.table
 
-fit_zeroinfl_normal <- function(covparams, covlink = NA, covname, obs_data, j){
+fit_zeroinfl_normal <- function(covparams, covlink = NA, covname, obs_data, j,
+                                model_fits){
   covmodels <- covparams$covmodels
   if (!is.null(covparams$covlink)){
     covlink <- covparams$covlink
@@ -94,20 +118,39 @@ fit_zeroinfl_normal <- function(covparams, covlink = NA, covname, obs_data, j){
   obs_data[obs_data[[covname]] != 0][, paste("log_", covname, sep = "")] <-
     log(obs_data[obs_data[[covname]] != 0][[covname]])
   # Fit binomial model on indicator of whether covariate is 0 or not
-  fit1 <- stats::glm(stats::as.formula(paste("I_", covmodels[j], sep = "")), family = stats::binomial(),
-              data = obs_data)
+  if (!is.null(covparams$control) && !is.na(covparams$control[j]) &&
+      !is.null(covparams$control[j][[1]])){
+    fit1 <- stats::glm(stats::as.formula(paste("I_", covmodels[j], sep = "")), family = stats::binomial(),
+                       control = covparams$control[j][[1]], data = obs_data)
+  } else {
+    fit1 <- stats::glm(stats::as.formula(paste("I_", covmodels[j], sep = "")), family = stats::binomial(),
+                       data = obs_data)
+  }
+
 
   # Fit Gaussian model on data points for which covariate does not equal 0
-  fit2 <- stats::glm(stats::as.formula(paste("log_", covmodels[j], sep = "")),
-              family = eval(parse(text = famtext)),
-              data = obs_data[obs_data[[covname]] != 0])
+  if (!is.null(covparams$control) && !is.na(covparams$control[j]) &&
+      !is.null(covparams$control[j][[2]])){
+    fit2 <- stats::glm(stats::as.formula(paste("log_", covmodels[j], sep = "")),
+                       family = eval(parse(text = famtext)),
+                       control = covparams$control[j][[2]],
+                       data = obs_data[obs_data[[covname]] != 0])
+  } else {
+    fit2 <- stats::glm(stats::as.formula(paste("log_", covmodels[j], sep = "")),
+                       family = eval(parse(text = famtext)),
+                       data = obs_data[obs_data[[covname]] != 0])
+  }
 
   fit1$rmse <- add_rmse(fit1)
   fit2$rmse <- add_rmse(fit2)
   fit1$stderr <- add_stderr(fit1)
   fit2$stderr <- add_stderr(fit2)
-  fit1 <- trim_glm(fit1)
-  fit2 <- trim_glm(fit2)
+  fit1$vcov <- add_vcov(fit1)
+  fit2$vcov <- add_vcov(fit2)
+  if (!model_fits){
+    fit1 <- trim_glm(fit1)
+    fit2 <- trim_glm(fit2)
+  }
 
   return (list(fit1, fit2))
 }
@@ -125,11 +168,13 @@ fit_zeroinfl_normal <- function(covparams, covlink = NA, covname, obs_data, j){
 #' @param covname     Name of the covariate at index \eqn{j}.
 #' @param obs_data    Data on which the model is fit.
 #' @param j           Integer specifying the index of the covariate.
+#' @param model_fits  Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #' @return            Fitted model for the covariate at index \eqn{j}.
 #' @keywords internal
 #' @import data.table
 
-fit_bounded_continuous <- function(covparams, covlink = NA, covname, obs_data, j){
+fit_bounded_continuous <- function(covparams, covlink = NA, covname, obs_data,
+                                   j, model_fits){
 
   covmodels <- covparams$covmodels
   if (!is.null(covparams$covlink)){
@@ -139,15 +184,29 @@ fit_bounded_continuous <- function(covparams, covlink = NA, covname, obs_data, j
   obs_data[, paste("norm_", covname, sep = "")] <-
     (obs_data[[covname]] - min(obs_data[[covname]]))/(max(obs_data[[covname]]) - min(obs_data[[covname]]))
   if (!is.na(covlink[j])){
-    fit <- stats::glm(stats::as.formula(paste("norm_", covmodels[j], sep = "")),
-               family = stats::gaussian(link = covlink[j]), data = obs_data, y = TRUE)
+    if (!is.null(covparams$control) && !is.na(covparams$control[j])){
+      fit <- stats::glm(stats::as.formula(paste("norm_", covmodels[j], sep = "")),
+                        family = stats::gaussian(link = covlink[j]), data = obs_data, y = TRUE,
+                        control = covparams$control[j])
+    } else {
+      fit <- stats::glm(stats::as.formula(paste("norm_", covmodels[j], sep = "")),
+                        family = stats::gaussian(link = covlink[j]), data = obs_data, y = TRUE)
+    }
   } else {
-    fit <- stats::glm(stats::as.formula(paste("norm_", covmodels[j], sep = "")), family = stats::gaussian(),
-               data = obs_data, y = TRUE)
+    if (!is.null(covparams$control) && !is.na(covparams$control[j])){
+      fit <- stats::glm(stats::as.formula(paste("norm_", covmodels[j], sep = "")), family = stats::gaussian(),
+                        data = obs_data, y = TRUE, control = covparams$control[j])
+    } else {
+      fit <- stats::glm(stats::as.formula(paste("norm_", covmodels[j], sep = "")), family = stats::gaussian(),
+                        data = obs_data, y = TRUE)
+    }
   }
   fit$rmse <- add_rmse(fit)
   fit$stderr <- add_stderr(fit)
-  fit <- trim_glm(fit)
+  fit$vcov <- add_vcov(fit)
+  if (!model_fits){
+    fit <- trim_glm(fit)
+  }
   return (fit)
 }
 
@@ -161,18 +220,30 @@ fit_bounded_continuous <- function(covparams, covlink = NA, covname, obs_data, j
 #'                    model statement, family, link function, etc.).
 #' @param obs_data    Data on which the model is fit.
 #' @param j           Integer specifying the index of the covariate.
+#' @param model_fits  Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #' @return            Fitted model for the covariate at index \eqn{j}.
 #' @keywords internal
 
-fit_trunc_normal <- function(covparams, obs_data, j){
+fit_trunc_normal <- function(covparams, obs_data, j, model_fits){
   covmodels <- covparams$covmodels
   point <- covparams$point[j]
   direction <- covparams$direction[j]
-  fit <- truncreg::truncreg(stats::as.formula(paste(covmodels[j])), data = obs_data, point = point,
-                            direction = direction, y = TRUE)
+  if (!is.null(covparams$control) && !is.na(covparams$control[j])){
+    args <- c(list(formula = stats::as.formula(paste(covmodels[j])),
+                   data = obs_data, point = point, direction = direction,
+                   y = TRUE), covparams$control[j])
+    fit <- do.call(truncreg::truncreg, args = args)
+  } else {
+    fit <- truncreg::truncreg(stats::as.formula(paste(covmodels[j])), data = obs_data, point = point,
+                              direction = direction, y = TRUE)
+  }
+
   fit$rmse <- add_rmse(fit)
   fit$stderr <- add_stderr(fit)
-  fit <- trim_truncreg(fit)
+  fit$vcov <- add_vcov(fit)
+  if (!model_fits){
+    fit <- trim_truncreg(fit)
+  }
   return (fit)
 }
 
@@ -202,19 +273,20 @@ fit_trunc_normal <- function(covparams, obs_data, j){
 #'                        third entry. The default is \code{NA}.
 #' @param time_name       Character string specifying the name of the time variable in \code{obs_data}.
 #' @param obs_data        Data on which the models are fit.
+#' @param model_fits      Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #' @return                A list of fitted models, one for each covariate in \code{covnames}.
 #' @keywords internal
 #' @import data.table
 
 pred_fun_cov <- function(covparams, covnames, covtypes, covfits_custom,
-                         restrictions, time_name, obs_data){
+                         restrictions, time_name, obs_data, model_fits){
   if (!is.na(restrictions[[1]][[1]])){ # Check for restrictions
     # Create list of covariates whose modeling is affected by restrictions
-    restrictnames <- lapply(1:length(restrictions), FUN = function(r){
+    restrictnames <- lapply(seq_along(restrictions), FUN = function(r){
       restrictions[[r]][[1]]
     })
     # Create list of conditions where covariates are modeled
-    conditions <- lapply(1:length(restrictions), FUN = function(r){
+    conditions <- lapply(seq_along(restrictions), FUN = function(r){
       restrictions[[r]][[2]]
     })
   } else {
@@ -225,7 +297,7 @@ pred_fun_cov <- function(covparams, covnames, covtypes, covfits_custom,
 
   subdata <- obs_data[obs_data[[time_name]] > 0]
 
-  fits <- lapply(1:length(covnames), FUN = function(j){
+  fits <- lapply(seq_along(covnames), FUN = function(j){
     if (!is.na(restrictions[[1]][[1]])){
       if (covnames[j] %in% restrictnames){
         i <- which(restrictnames %in% covnames[j])
@@ -263,19 +335,26 @@ pred_fun_cov <- function(covparams, covnames, covtypes, covfits_custom,
       }
     }
     if (covtypes[j] == 'binary'){
-      fit_glm(covparams = covparams, covfam = 'binomial', obs_data = subdata, j = j)
+      fit_glm(covparams = covparams, covfam = 'binomial', obs_data = subdata,
+              j = j, model_fits = model_fits)
     } else if (covtypes[j] == 'normal'){
-      fit_glm(covparams = covparams, covfam = 'gaussian', obs_data = subdata, j = j)
+      fit_glm(covparams = covparams, covfam = 'gaussian', obs_data = subdata,
+              j = j, model_fits = model_fits)
     } else if (covtypes[j] == 'categorical'){
-      fit_multinomial(covparams, obs_data = subdata, j = j)
+      fit_multinomial(covparams, obs_data = subdata, j = j,
+                      model_fits = model_fits)
     } else if (covtypes[j] == 'zero-inflated normal'){
-      fit_zeroinfl_normal(covparams, covname = covnames[j], obs_data = subdata, j = j)
+      fit_zeroinfl_normal(covparams, covname = covnames[j], obs_data = subdata,
+                          j = j, model_fits = model_fits)
     } else if (covtypes[j] == 'bounded normal'){
-      fit_bounded_continuous(covparams, covname = covnames[j], obs_data = subdata, j = j)
+      fit_bounded_continuous(covparams, covname = covnames[j],
+                             obs_data = subdata, j = j, model_fits = model_fits)
     } else if (covtypes[j] == 'truncated normal'){
-      fit_trunc_normal(covparams = covparams, obs_data = subdata, j = j)
+      fit_trunc_normal(covparams = covparams, obs_data = subdata, j = j,
+                       model_fits = model_fits)
     } else if (covtypes[j] == 'custom'){
-      covfits_custom[[j]](covparams, covname = covnames[j], obs_data = subdata, j = j)
+      covfits_custom[[j]](covparams, covname = covnames[j], obs_data = subdata,
+                          j = j)
     }
   })
   return (fits)
@@ -295,11 +374,13 @@ pred_fun_cov <- function(covparams, covnames, covtypes, covfits_custom,
 #' @param outcome_name   Character string specifying the name of the outcome variable in \code{obs_data}.
 #' @param time_name      Character string specifying the name of the time variable in \code{obs_data}.
 #' @param obs_data       Data on which the model is fit.
+#' @param model_fits     Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #' @return               Fitted model for the outcome variable.
 #' @keywords internal
 #' @import data.table
 
-pred_fun_Y <- function(model, yrestrictions, outcome_type, outcome_name, time_name, obs_data){
+pred_fun_Y <- function(model, yrestrictions, outcome_type, outcome_name,
+                       time_name, obs_data, model_fits){
   if (outcome_type == 'continuous' || outcome_type == 'continuous_eof'){
     outcome_fam <- stats::gaussian()
   } else if (outcome_type == 'survival' || outcome_type == 'binary_eof'){
@@ -334,7 +415,10 @@ pred_fun_Y <- function(model, yrestrictions, outcome_type, outcome_name, time_na
   }
   fitY$rmse <- add_rmse(fitY)
   fitY$stderr <- add_stderr(fitY)
-  fitY <- trim_glm(fitY)
+  fitY$vcov <- add_vcov(fitY)
+  if (!model_fits){
+    fitY <- trim_glm(fitY)
+  }
   return (fitY)
 }
 
@@ -350,11 +434,12 @@ pred_fun_Y <- function(model, yrestrictions, outcome_type, outcome_name, time_na
 #'                               the competing event variable takes on the value in the
 #'                               second entry.
 #' @param obs_data               Data on which the model is fit.
+#' @param model_fits             Logical scalar indicating whether to return the fitted models. The default is \code{FALSE}.
 #' @return                       Fitted model for the competing event variable.
 #' @keywords internal
 #' @import data.table
 
-pred_fun_D <- function(model, compevent_restrictions, obs_data){
+pred_fun_D <- function(model, compevent_restrictions, obs_data, model_fits){
   if (!is.na(compevent_restrictions[[1]][[1]])){ # Check for restrictions on compevent event variable modeling
     # Set condition where competing event variable is modeled
     condition <- compevent_restrictions[[1]][1]
@@ -375,6 +460,9 @@ pred_fun_D <- function(model, compevent_restrictions, obs_data){
   }
   fitD$rmse <- add_rmse(fitD)
   fitD$stderr <- add_stderr(fitD)
-  fitD <- trim_glm(fitD)
+  fitD$vcov <- add_vcov(fitD)
+  if (!model_fits){
+    fitD <- trim_glm(fitD)
+  }
   return (fitD)
 }
